from django.contrib import admin

from nanolog.models import Nanolog


class NanologAdmin(admin.ModelAdmin):
    list_display = ('log_type', 'details', 'note', 'created_date', 'ip', 'user', 'content_object', )
    search_fields = ('log_type', 'details', 'note', 'ip', 'user__username', 'user__first_name', 'user__last_name', )
    list_filter = ('log_type', )
    date_hierarchy = 'created_date'


admin.site.register(Nanolog, NanologAdmin)



from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import validate_ipv46_address
from nanolog.models import Nanolog


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[-1].strip()
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def nanolog(log_type, details, note=None, user=None, ip=None, log_object=None, request=None):
    """
    Write into Nanolog the received data.

    <log_type>  - string
    <details>   - string
    <note>      - string
    <user>      - user instance or None
    <ip>        - string
    <log_object>- string
    <request>   - request object

    """
    if not log_type or not details:
        return
    if not user and request:
        user = request.user
    values = dict(
                user=user,
                log_type=log_type,
                details=details,
                note=note,
        )
    print(ip)
    if not ip and request:
        ip = get_client_ip(request)
    print(ip)
    try:
        validate_ipv46_address(ip)
        values['ip'] = ip
    except (ValidationError, TypeError):
        values['ip'] = None
    if log_object and hasattr(log_object, 'pk'):
        values['content_object'] = log_object
    Nanolog.objects.create(**values)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User


class Nanolog(models.Model):
    user = models.ForeignKey(User, verbose_name=_(u"User"), blank=True, null=True, on_delete=models.SET_NULL, db_index=True)
    log_type = models.CharField(_('Log type'), max_length=255, db_index=True)
    details = models.CharField(_('Details'), max_length=255, db_index=True)
    ip = models.GenericIPAddressField(_('Ip'), null=True, blank=True, db_index=True)
    note = models.TextField(_('Note'), null=True, blank=True)

    content_type = models.ForeignKey(ContentType, null=True, blank=True)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    created_date = models.DateTimeField(_('created date'), auto_now_add=True, db_index=True)

    class Meta:
        ordering = ['-created_date']

    def __unicode__(self):
        values = ['nanolog', unicode(self.log_type), unicode(self.created_date)]
        if self.user:
            values.insert(2, self.user.username)
        return " - ".join(values)



from django.test import TestCase
from nanolog.models import Nanolog
from nanolog.utils import nanolog
from django.contrib.auth.models import User


class SimpleTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(
                username='test', email='test@test', password='test')


    def test_nanolog(self):
        logs = Nanolog.objects.all()
        self.assertEquals(logs.count(), 0)

        nanolog('access', 'this_area')
        self.assertEquals(logs.count(), 1)
        self.assertEquals(logs[0].user, None)
        self.assertEquals(logs[0].log_type, 'access')
        self.assertEquals(logs[0].details, 'this_area')
        self.assertEquals(logs[0].note, None)
        self.assertEquals(logs[0].ip, None)
        self.assertEquals(logs[0].content_object, None)

        nanolog('access', '')
        self.assertEquals(logs.count(), 1)

        nanolog('', 'this_area')
        self.assertEquals(logs.count(), 1)

        nanolog('', '')
        self.assertEquals(logs.count(), 1)

        nanolog('changed_password', 'from:x|to:y', 'as we sugested in the newsletter')
        self.assertEquals(logs.count(), 2)
        self.assertEquals(logs[0].user, None)
        self.assertEquals(logs[0].log_type, 'changed_password')
        self.assertEquals(logs[0].details, 'from:x|to:y')
        self.assertEquals(logs[0].note, 'as we sugested in the newsletter')
        self.assertEquals(logs[0].ip, None)
        self.assertEquals(logs[0].content_object, None)

        nanolog('changed_password', 'from:x|to:y', 'as we sugested in the newsletter', self.user)
        self.assertEquals(logs.count(), 3)
        self.assertEquals(logs[0].user, self.user)
        self.assertEquals(logs[0].log_type, 'changed_password')
        self.assertEquals(logs[0].details, 'from:x|to:y')
        self.assertEquals(logs[0].note, 'as we sugested in the newsletter')
        self.assertEquals(logs[0].ip, None)
        self.assertEquals(logs[0].content_object, None)

        nanolog('changed_password', 'from:x|to:y', 'as we sugested in the newsletter', self.user, '127.0.0.1')
        self.assertEquals(logs.count(), 4)
        self.assertEquals(logs[0].user, self.user)
        self.assertEquals(logs[0].log_type, 'changed_password')
        self.assertEquals(logs[0].details, 'from:x|to:y')
        self.assertEquals(logs[0].note, 'as we sugested in the newsletter')
        self.assertEquals(logs[0].ip, '127.0.0.1')
        self.assertEquals(logs[0].content_object, None)

        nanolog('changed_password', 'from:x|to:y', 'as we sugested in the newsletter', self.user, 'wr.on.g.ip')
        self.assertEquals(logs.count(), 5)
        self.assertEquals(logs[0].user, self.user)
        self.assertEquals(logs[0].log_type, 'changed_password')
        self.assertEquals(logs[0].details, 'from:x|to:y')
        self.assertEquals(logs[0].note, 'as we sugested in the newsletter')
        self.assertEquals(logs[0].ip, None)
        self.assertEquals(logs[0].content_object, None)

        nanolog('changed_password', 'from:x|to:y', 'as we sugested in the newsletter', self.user, '127.0.0.1', 'wrong_object')
        self.assertEquals(logs.count(), 6)
        self.assertEquals(logs[0].user, self.user)
        self.assertEquals(logs[0].log_type, 'changed_password')
        self.assertEquals(logs[0].details, 'from:x|to:y')
        self.assertEquals(logs[0].note, 'as we sugested in the newsletter')
        self.assertEquals(logs[0].ip, '127.0.0.1')
        self.assertEquals(logs[0].content_object, None)

        nanolog('changed_password', 'from:x|to:y', 'as we sugested in the newsletter', self.user, '127.0.0.1', self.user)
        self.assertEquals(logs.count(), 7)
        self.assertEquals(logs[0].user, self.user)
        self.assertEquals(logs[0].log_type, 'changed_password')
        self.assertEquals(logs[0].details, 'from:x|to:y')
        self.assertEquals(logs[0].note, 'as we sugested in the newsletter')
        self.assertEquals(logs[0].ip, '127.0.0.1')
        self.assertEquals(logs[0].content_object, self.user)
